import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The players that are controlled in the painball world by the two seperate players
 * 
 * @author Stephen Clement 
 * @version 1.1
 */
public class Soldier extends MovingUnit
{
    /**
     * Act - do whatever the Soldiers wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public int team = 1;
    private int width = 7;
    private int height = 7;
    private int delay = 20;
    public int reload = 0;
    public int ammo = 63;
    public int bullets = 7;
    public int clipSize = 7;
    public int clips = 9;
    private GreenfootImage pistol;
    private GreenfootImage machineGun;
    private GreenfootImage shotgun;
    private int health = 6;
    private int wait = 0;
    private int splatWidth = 0;
    private int splatHeight = 0;
    private int currentWeapon = 1;
    private String upKey = "up";
    private String downKey = "down";
    private String leftKey = "left";
    private String rightKey = "right";
    private String shootKey = "space";
    private String rotateLeft = ",";
    private String rotateRight = ".";
    private String pickUp = "k";
    private int rotationSpeed = 5;
    private int pickUpDelay = 0;
    private gun gun;
    private int gunName;
    public boolean reloading = false;
    public int reloadTime;
    private score_counter clipsAmount = new score_counter("clips :");
    private score_counter bulletsAmount = new score_counter("bullets :");
    
    public void act() 
    {
        seeObstacles();
        setParametersForCurrentWeapon();
        shoot();
        rotate();
        decrementAllDelays();
        pickUpWeapon();
        ammoControl();
        ammoCounter();
    } 
        
    public void setTeam(int team)
    {
        this.team = team;
        
        if (this.team == 2){
            this.upKey = "up";
            this.downKey = "down";
            this.leftKey = "left";
            this.rightKey = "right";
            this.shootKey = "enter";
            this.rotateLeft = "-";
            this.rotateRight = "=";
            this.pickUp = "0";
        }
        else{
            this.upKey = "w";
            this.downKey = "s";
            this.leftKey = "a";
            this.rightKey = "d";
            this.shootKey = "space";
            this.rotateLeft = "g";
            this.rotateRight = "h";
            this.pickUp = "j";
        }
    }   
    
    /**
     * changes the weapon to whatever the soldier is currently carrying
     */
    public void setParametersForCurrentWeapon()
    {        
         switch(currentWeapon){
            case 1 :
                    if(team == 1){
                         pistol = new GreenfootImage("Soldier_Blue_w_pistol.PNG");
                    }
                    else if(team == 2)
                    {
                        pistol = new GreenfootImage("Soldier_Red_w_pistol.PNG");
                    }
                    setImage(pistol);
                    width = 7;
                    height = 7;
                    splatWidth = 15;
                    splatHeight = 15;
                    delay = 20;
                    clipSize = 7;
                    reloadTime = 50;
                    break;
                    
            case 2 :
                    if(team == 1){
                        machineGun = new GreenfootImage("Soldier_Blue_w_machine_gun.PNG");
                    }
                    else if (team == 2)
                    {
                        machineGun = new GreenfootImage("Soldier_Red_w_machine_gun.PNG");
                    }
                    setImage(machineGun);
                    width = 9;
                    height = 9;
                    splatWidth = 25;
                    splatHeight = 25;
                    delay = 10;
                    clipSize = 15;
                    reloadTime = 50;
                    break;
                    
            case 3 :
                    if(team == 1){
                        shotgun = new GreenfootImage("Soldier_Blue_w_shotgun.PNG");
                    }
                    else if(team == 2)
                    {
                        shotgun = new GreenfootImage("Soldier_Red_w_shotgun.PNG");
                    }
                    setImage(shotgun);
                    width = 5;
                    height = 5;
                    splatWidth = 25;
                    splatHeight = 25;
                    delay = 30;
                    clipSize = 4;
                    reloadTime = 100;
                    break;
                
        }
    }
    
    /**
     * This checks to make sure that the soldier is not moving through obstacles
     */
    public void seeObstacles(){
        if(getObjectsAtOffset(0,-getHeight()/2,sandbag.class).size() == 0&&getY()>getHeight()/2){
        if(Greenfoot.isKeyDown(upKey)){
            moveUp();
        }
    }
     if(getObjectsAtOffset(0,getHeight()/2,sandbag.class).size() == 0&&getObjectsAtOffset(0,-getHeight()/2,black.class).size() == 0&&getY()<getWorld().getHeight()-getHeight()/2-100){
        if(Greenfoot.isKeyDown(downKey)){
            moveDown();
        }
    }
     if(getObjectsAtOffset(-getWidth()/2,0,sandbag.class).size() == 0 && getX() > getWidth()/2 +5 ){
        if(Greenfoot.isKeyDown(leftKey)){
            moveLeft();
        }
    }
     if(getObjectsAtOffset(getWidth()/2,0,sandbag.class).size() == 0 && getX() < getWorld().getWidth()- getWidth()/2 - 5){
        if(Greenfoot.isKeyDown(rightKey)){
            moveRight();
        }
    }
}
    
    /**
     * subtracts health from the soldier
     */
     public void subHealth(int h){
        health = health - h;
    }
    
    /**
     * Shots a paintball
     */
    public void shoot(){
        if(Greenfoot.isKeyDown(shootKey)&&wait == 0&& bullets > 0){
            Greenfoot.playSound("paintball_shot.wav");
            getWorld().addObject(new Paintball(team,width,height,getRotation(),splatWidth,splatHeight),getX(),getY());
            bullets--;
            if(currentWeapon == 3)
            {
                getWorld().addObject(new Paintball(team,width,height,getRotation()-3,splatWidth,splatHeight),getX(),getY());
                 getWorld().addObject(new Paintball(team,width,height,getRotation()+3,splatWidth,splatHeight),getX(),getY());
            }
            wait = wait + delay;
        }
    }
    
    
    /**
     * timer for gun delay
     */
    public void decrementAllDelays()
    {
        if(wait > 0)
        {
            wait--;
        }
        
        if(pickUpDelay > 0)
        {
            pickUpDelay --;
        }
        
        if(reload > 0)
        {
            reload--;
        }
    }
    
    /**
     * rotation controls
     */
    public void rotate()
    {
        if(Greenfoot.isKeyDown(rotateLeft))
        {
            setRotation(getRotation()-rotationSpeed);
        }
        else if (Greenfoot.isKeyDown(rotateRight))
        {
            setRotation(getRotation()+rotationSpeed);
        }
    }
    
    /**
     * picks up the weapon in the current position upon key press
     */
    public void pickUpWeapon()
    {
        if(getOneObjectAtOffset(0,0,gun.class) != null)
        {
            gun = (gun)getOneObjectAtOffset(0,0,gun.class);
            gunName = gun.typeOfGun;
        }
        else
        {
            gun = null;
        }
        
        if( Greenfoot.isKeyDown(pickUp) && pickUpDelay == 0 && gun != null)
        {
        switch(gunName){
            case 1 :
                    pickUpDelay = 10;
                    this.ammo = gun.ammo;
                    this.clips = gun.clips;
                    this.bullets = gun.bullets;
                    getWorld().removeObject(gun);
                    dropWeapon(currentWeapon);
                    currentWeapon = 1;
                    break;
                    
            case 2 :
                    pickUpDelay = 10;
                    this.ammo = gun.ammo;
                    this.clips = gun.clips;
                    this.bullets = gun.bullets;
                    getWorld().removeObject(gun);
                    dropWeapon(currentWeapon);
                    currentWeapon = 2;
                    break;
                    
            case 3 :
                    pickUpDelay = 10;
                    this.ammo = gun.ammo;
                    this.clips = gun.clips;
                    this.bullets = gun.bullets;
                    getWorld().removeObject(gun);
                    dropWeapon(currentWeapon);
                    currentWeapon = 3;
                    break;
                }
                Greenfoot.playSound("pick_up_weapon.wav");
            }
        }
        
        
        /**
         * drops the current weapon
         */
        public void dropWeapon(int currentWeapon)
        {
                getWorld().addObject(new gun(currentWeapon,ammo,clips,bullets), getX(),getY());
        }
                
         /**
          * a sequence of events to occur for the soldiers death
          */
         public void die()
         {
             dropWeapon(currentWeapon);
             ((StephenWorld) getWorld()).addPoint(team);
             getWorld().removeObject(clipsAmount);
             getWorld().removeObject(bulletsAmount);
            }
            
         /**
          * changes the ammo stats and stuff
          */
         public void ammoControl()
         {
             if(bullets == 0&& ammo > 0&& reloading == false)
             {
                 Greenfoot.playSound("reloading.wav");
                 reloading = true;
                 reload = reloadTime;
             }
                if (reload == 1)
                {
                    bullets = clipSize;
                    ammo = ammo - clipSize;
                    clips--;
                    reloading = false;
                }
    }
    
    /**
     * creates a counter that keeps track of ammo
     */
    public void ammoCounter()
    {
        if(team == 2)
        {
            getWorld().addObject(clipsAmount, 700, 550);
            getWorld().addObject(bulletsAmount, 700, 570);
        }
        else
        {
            getWorld().addObject(clipsAmount, 50, 550);
            getWorld().addObject(bulletsAmount, 50, 570);
        }
        clipsAmount.setScore(clips);
        bulletsAmount.setScore(bullets);
    }
        
}
